'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fetch = _interopDefault(require('node-fetch'));
var $ = _interopDefault(require('cheerio'));
var url = _interopDefault(require('url'));
var FormData = _interopDefault(require('form-data'));
var vm = require('vm');
var toughCookie = require('tough-cookie');
var TempMail = require('temp-mail');
var TempMail__default = TempMail['default'];
var debug = _interopDefault(require('debug'));
var fs = _interopDefault(require('fs'));
var child = _interopDefault(require('child_process'));
var path = _interopDefault(require('path'));
var stream = require('stream');
var os = _interopDefault(require('os'));

function cheerio(options) {
	return req => req.text().then(body => $.load(body, options));
}

const ANONIMITY_LEVELS = ['transparent', 'anonymous', 'elite proxy'];

function scrap() {
	return fetch('http://www.free-proxy-list.net').then(cheerio()).then($$$1 => $$$1('#proxylisttable > tbody > tr').map((i, e) => {
		const element = $$$1(e).find('td');
		const ip = element.eq(0).text();
		const port = element.eq(1).text();
		const country = element.eq(3).text().toUpperCase();
		const anonymity = ANONIMITY_LEVELS.indexOf(element.eq(4).text());
		const type = element.eq(5).text() == 'yes' ? 'https' : 'http';
		return { ip, port, country, anonymity, type };
	}).get());
}

function createTextScraper(url$$1) {
	return function scrap() {
		return fetch(url$$1).then(res => res.text()).then(text => extractProxies(text));
	};
}

function extractProxies(text, data = {}) {
	return text.split('\n').map(proxy => proxy.split(':')).map(([ip, port]) => Object.assign({ ip, port: parseInt(port) }, data));
}

class Session {
	constructor(fetch$$1) {
		this.jar = new toughCookie.CookieJar();
		this._fetch = fetch$$1;
	}

	fetch(url$$1, opts = {}) {
		return this.getCookies(url$$1).then(cookies => {
			return this._fetch(url$$1, Object.assign(opts, {
				headers: Object.assign(opts.headers || {}, {
					cookie: cookies.join('; ')
				})
			})).then(res => {
				let cookies = res.headers.get('set-cookie');
				cookies = cookies && cookies.split(',') || [];
				cookies.map(cookie => this.jar.setCookie(cookie, url$$1, () => null));
				return res;
			});
		});
	}

	getCookies(url$$1) {
		return new Promise((resolve, reject) => {
			this.jar.getCookies(url$$1, (err, cookies) => {
				if (err) reject(err);else resolve(cookies);
			});
		});
	}
}

const log$2 = debug('proxy-scraper:temp-mail');

const NOOP = function NOOP() {};

function poll(email, interval = 5000, iterations = 20) {
	return new Promise((resolve, reject) => {
		let count = 0;
		const id = setInterval(() => {
			if (count < iterations) {
				count++;
				log$2('Polling inbox for mail %s iteration %d, max: %d', email, count, iterations);
				TempMail__default.getInbox(email).then(mails => {
					resolve(mails);
					clearInterval(id);
				}).catch(NOOP); //Ignore Error
			} else {
				clearInterval(id);
				reject(new Error('Timeout exceeded, no messages found.'));
			}
		}, interval);
	});
}

const log$1 = debug('proxy-scraper:gatherproxy.com');

const CAPTCHA_REPLACE = {
	multiplied: '*',
	plus: '+',
	minus: '-',
	x: '*',
	zero: 0,
	one: 1,
	two: 2,
	three: 3,
	four: 4,
	five: 5,
	six: 6,
	seven: 7,
	eight: 8,
	nine: 9
};

const ANONIMITY_LEVELS$1 = ['transparent', 'anonymous', 'elite'];

function scrap$1() {
	const session = new Session(fetch);
	const fetch$$1 = (url$$1, options) => session.fetch(url$$1, options);
	return getAccount(fetch$$1).then(() => {
		let chain = Promise.resolve([]);
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = ANONIMITY_LEVELS$1[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				let level = _step.value;

				chain = chain.then(data => getProxyListId(fetch$$1, level).then(id => downloadProxyList(fetch$$1, level, id)).then(text => data.push(extractProxies(text, {
					anonimity: ANONIMITY_LEVELS$1.indexOf(level)
				}))).then(() => data));
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		return chain.then(datas => datas.reduce((prev, next) => prev.concat(next), []));
	});
}

function getAccount(fetch$$1) {
	return loadAccountFromCache().then(({ email, password }) => {
		log$1('Loaded account with email %s and password %s from .gatherproxy.account', email, password);
		return login(fetch$$1, email, password);
	}).catch(() => {
		log$1('Invalid account in .gatherproxy.account, creating new one');
		return createAccount(fetch$$1).then(({ email, password }) => {
			log$1('Storing account with email %s and password %s in .gatherproxy.account', email, password);
			return storeAccountToCache(email, password).catch(() => ({
				email,
				password
			}));
		}).then(({ email, password }) => login(fetch$$1, email, password));
	});
}

function loadAccountFromCache() {
	return new Promise((resolve, reject) => {
		fs.readFile('./.gatherproxy.account', 'utf8', (err, data) => {
			if (err) reject(err);else resolve(JSON.parse(data));
		});
	});
}

function storeAccountToCache(email, password) {
	return new Promise((resolve, reject) => {
		fs.writeFile('./.gatherproxy.account', JSON.stringify({ email, password }), err => {
			if (err) reject(err);else resolve({ email, password });
		});
	});
}

function createAccount(fetch$$1) {
	const email = `${Math.random().toString(36).substring(7)}@doanart.com`; //@binka.me don't receive email
	log$1('Creating account with email %s', email);
	const form = new FormData();
	form.append('email', email);
	return fetch$$1('http://www.gatherproxy.com/subscribe', {
		method: 'POST',
		body: form,
		headers: form.getHeaders()
	}).then(() => poll(email)).then(([{ mail_text_only }]) => /<p>Password: (.*)<\/p>/.exec(mail_text_only)[1]).then(password => {
		log$1('Account %s with password %s created', email, password);
		return { email, password };
	});
}

function login(fetch$$1, email, password) {
	log$1('Logging in with email %s and password %s', email, password);
	return fetch$$1('http://www.gatherproxy.com/subscribe/login').then(cheerio()).then($$$1 => {
		const captcha = $$$1('#body > form > .label > .blue').first().text();

		const form = new FormData();
		form.append('Username', email);
		form.append('Password', password);
		form.append('Captcha', solveCaptcha(captcha));

		return fetch$$1('http://www.gatherproxy.com/subscribe/login', {
			method: 'POST',
			body: form,
			headers: form.getHeaders()
		});
	});
}

function solveCaptcha(raw) {
	const captcha = raw.split(' ').map(part => part.toLowerCase()).map(part => part in CAPTCHA_REPLACE ? CAPTCHA_REPLACE[part] : part);

	captcha.splice(captcha.indexOf('='), 1); // Remove the =

	const c = captcha.join(' ');
	const result = new vm.Script(c).runInNewContext({});

	log$1('Captcha %s (raw: %s) solved, result: %d', c, raw, result);
	return result;
}

function getProxyListId(fetch$$1, anonimity) {
	log$1('Getting id for anonimity %s', anonimity);
	const form = new FormData();
	form.append('Uptime', 0);
	form.append('Type', anonimity);
	return fetch$$1('http://www.gatherproxy.com/proxylist/anonymityplaintext', {
		method: 'POST',
		body: form,
		headers: form.getHeaders(),
		redirect: 'manual'
	}).then(res => url.parse(res.headers.get('location'), true).query.sid);
}

function downloadProxyList(fetch$$1, anonimity, id) {
	log$1('Downloading proxy with anonimity %s using id %s', anonimity, id);
	const form = new FormData();
	form.append('ID', id);
	form.append('T', anonimity);
	return fetch$$1('http://www.gatherproxy.com/proxylist/downloadproxylist/', {
		method: 'POST',
		body: form,
		headers: form.getHeaders()
	}).then(res => res.text());
}

let maxtime = 1000;

function scrap$2() {
	return fetch(`https://hidemy.name/en/proxy-list/?maxtime=${maxtime}&type=hs`, {
		headers: {
			'User-Agent': 'Mozilla/5.0'
		}
	}).then(cheerio()).then($$$1 => {
		return $$$1('.proxy__t > tbody > tr').map((i, e) => {
			const element = $$$1(e).find('td');
			const ip = $$$1(element).find('td').eq(0).text();
			const port = $$$1(element).find('td').eq(1).text();
			return { ip, port };
		}).get();
	});
}

const ANONIMITY_LEVELS$2 = ['transparent', 'anonymous', 'elite'];

function scrap$3(types) {
	return fetch('https://hidester.com/proxydata/php/data.php?mykey=csv&gproxy=2', {
		headers: {
			Referer: 'https://hidester.com/proxylist/'
		}
	}).then(req => req.json()).then(datas => datas.map(({ IP: ip, PORT: port, type, country, anonymity }) => ({
		ip,
		port,
		type,
		country,
		anonimity: ANONIMITY_LEVELS$2.indexOf(anonymity.toLowerCase())
	})));
}

var multiproxy = createTextScraper('http://multiproxy.org/txt_all/proxy.txt');

function scrap$4() {
	const session = new Session(fetch);

	return session.fetch('https://nordvpn.com/free-proxy-list/').then(() => session.fetch('https://nordvpn.com/wp-admin/admin-ajax.php?searchParameters%5B0%5D%5Bname%5D=proxy-country&searchParameters%5B0%5D%5Bvalue%5D=&searchParameters%5B1%5D%5Bname%5D=proxy-ports&searchParameters%5B1%5D%5Bvalue%5D=&offset=0&limit=100000&action=getProxies', {
		method: 'POST'
	})).then(res => res.json()).then(datas => datas.map(({ ip, port, country, type }) => ({
		ip,
		port,
		country,
		type: type.toLowerCase()
	})));
}

const SOURCES = [{
	url: 'http://proxyserverlist-24.blogspot.fr/',
	type: 'http',
	selector: '.post-body > pre > span > span:nth-child(2)'
}, {
	url: 'http://sslproxies24.blogspot.fr/',
	type: 'https',
	selector: '.post-body > pre > span > span'
}, {
	url: 'http://vip-socks24.blogspot.com/',
	type: 'socks5',
	selector: '.post-body > textarea'
}];

function scrap$5() {
	const result = [];
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = SOURCES[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			const source = _step.value;

			result.push(fetch(source.url).then(cheerio()).then($$$1 => $$$1('.post-title > a').eq(0).attr('href')) //Only take first
			.then(url$$1 => fetch(url$$1)).then(cheerio()).then($$$1 => $$$1(source.selector).eq(0).text()).then(proxies => extractProxies(proxies, { type: source.type })));
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return Promise.all(result).then(datas => datas.reduce((prev, next) => prev.concat(next), []));
}

function scrap$6() {
	let promises = [];
	for (let offset = 0; offset < 1000; offset += 50) {
		promises.push(fetch(`http://proxydb.net/?limit=50&offset=${offset}`).then(cheerio()).then($$$1 => {
			if (!$$$1) return [];
			return $$$1('table > tbody > tr').map((i, element) => {
				let url$$1 = $$$1(element).find('td > a').first().text().trim();
				let ip = /(\d+\.){3}\d+/.exec(url$$1)[0];
				let port = /\d+$/.exec(url$$1)[0];

				return { ip, port };
			}).get();
		}));
	}
	return Promise.all(promises).then(values => values.reduce((prev, next) => prev.concat(next)), []);
}

var proxyspy = createTextScraper('http://txt.proxyspy.net/proxy.txt');

//TODO: Better Text Scrapper

var scrapers = {
	'free-proxy-list.net': scrap,
	'gatherproxy.com': scrap$1,
	'hidemy.name': scrap$2,
	'hidester.com': scrap$3,
	'multiproxy.org': multiproxy,
	'nordvpn.com': scrap$4,
	'proxy24.blogspot.fr': scrap$5,
	'proxydb.net': scrap$6,
	'proxyspy.net': proxyspy
	//"spys.me": spys
};

class Lock {
	constructor(inner) {
		this.inner = Promise.resolve(inner);
	}

	get(callback) {
		const p = this.inner.then(inner => {
			const result = callback(inner);
			return Promise.resolve(result).then(result => ({ inner, result }), err => ({ inner, err }));
		});
		this.inner = p.then(({ inner }) => inner);
		return p.then(({ result, err }) => err ? Promise.reject(err) : result);
	}
}

const log = debug('proxy-scraper');

class ProxyScraper {
	constructor({ workerCount = os.cpus().length } = {}) {
		this._workers = [];
		for (let i = 0; i < workerCount; i++) {
			log('Spawning worker %d', i);
			const worker = child.fork(path.join(__dirname, './src/worker.js'), [i]);
			worker.on('error', error => console.error(error));
			this._workers.push(new Lock(worker));
		}
	}

	getProxies(timeout) {
		return this.scrapProxies().then(proxies => this.testProxies(timeout, proxies));
	}

	testProxies(timeout, proxies) {
		const stream$$1 = new stream.Readable({ objectMode: true });
		const proxiesCount = proxies.length;
		const queue = proxies.slice(0); //Clone it
		stream$$1._read = () => {
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this._workers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					const worker = _step.value;

					let done = false;
					const run = () => {
						if (queue.length) {
							const proxy = queue.pop();
							worker.get(worker => {
								const p = this._testProxy({
									url: 'http://example.com/',
									proxy: proxy.port == 443 ? `https://${proxy.ip}` : `http://${proxy.ip}:${proxy.port}`,
									timeout
								}, worker);
								return p;
							}).then(time => {
								done = true;
								proxy.time = time;
								log('Working proxy: %o', proxy);
								stream$$1.push(proxy);
							}).catch(e => null).then(() => {
								stream$$1.emit('progress', {
									length: proxiesCount,
									remaining: queue.length,
									percentage: (1 - queue.length / proxiesCount) * 100
								});
								return done ? null : run();
							});
						} else {
							stream$$1.push(null);
						}
					};
					run();
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		};
		return stream$$1;
	}

	_testProxy(proxy, worker) {
		worker.send(proxy);
		return new Promise((resolve, reject) => {
			worker.once('message', data => {
				if (data.working) {
					resolve(data.time);
				} else {
					reject(data.error);
				}
			});
		});
	}

	scrapProxies() {
		const proxies = [];
		log('Scrapers: %o', Object.keys(ProxyScraper.scrapers));
		for (let scraper in ProxyScraper.scrapers) {
			proxies.push(scrapers[scraper]().then((proxies = []) => {
				log('Found %d proxies from %s', proxies.length, scraper);
				proxies.forEach(proxy => proxy.source = scraper);
				return proxies;
			}).catch(e => {
				log('Error while scraping proxies with %s\n%o', scraper, e);
				return [];
			}));
		}
		return Promise.all(proxies).then(values => values.reduce((prev, next) => prev.concat(next)), []);
	}
}

ProxyScraper.scrapers = scrapers;

module.exports = ProxyScraper;
//# sourceMappingURL=6.js.map
